/*----------------------------------------------------------------------------------
% Authors: V. Bianchi, M. Zampetti
%          A. Canessa, M. Chessa, F. Solari
% PSPC-lab - Biophysical and Electronic Engineering Department - University of Genoa
% September 2010
------------------------------------------------------------------------------------*/
#include <ipp.h>
#include <string>

#ifndef POPCOD_DISP2D_H
#define POPCOD_DISP2D_H

//----------------------------------------------------------
//A simple wrapper class for the population coding model
//of 2D disparity computation (developed in C & IIP)
//----------------------------------------------------------
class PopCod_Disp2D{
	int width;        //image width
	int height;       //image height
	int n_scales;     //number of scales
	int n_phase_s;	  //number of phase shifts
	float dmax;       //max detectable disparity value
	float energy_th;  //energy threshold
	int ori_thr;      //threshold on orientation (0 if not used)
public:
	PopCod_Disp2D(std::string filter_filename, int width, int height, int n_scales, int n_phase_s, float dmax, float energy_th, int ori_thr);
	~PopCod_Disp2D();
	void Compute_maps(const Ipp32f *left_img, const Ipp32f *right_img, Ipp32f *d_x, Ipp32f *d_y);
	//----------------------------------------------------------------------------------------------
	//-------------------------C & IPP code---------------------------------------------------------
	//----------------------------------------------------------------------------------------------
private:
	static const int undi=9999; //pixel value for unreliable disparity
	struct MatrixIpp
	{
		Ipp32f *ptr;   //image
		int m;		   //rows
		int n;         //columns
		int stp;       //stepbyte
	};
	//----------------------------------------------------------------------------------------------
	void img_ipp_alloc_frombin(const Ipp32f *imgbin, MatrixIpp *imgipp);
	void img_ipp_dealloc( MatrixIpp *img);
	void img_ipp_2bin(const MatrixIpp *img,Ipp32f *imgdst);
	void load_gabor_filters(std::string filter_filename);
	//----------------------------------------------------------------------------------------------
	struct MatrixIpp ******buff, ******copy_buff;
	struct MatrixIpp ***energy,  ****dec_energy, ***dec_population;
	struct MatrixIpp **population, **mask, **full_disparity, **copy_full;
	struct MatrixIpp *fil_spaz, *pyramsx, *pyramdx; 
	struct MatrixIpp filpyr, disparity, mask2, copymask2;    
	int m, n, n_liv, n_frame, n_vel, n_orient, dx_sx, n_phase,n_fil_spaz, nc_min;
	int taps;
	float energy_thr,fo,PI_greco;

	//----alloc--------------------------------------------------------------------------------
	void allocate_buffer(int n_liv_pir, int n_frame, int sx_dx, int n_orient, int n_phase, int m_imm, int n_imm, struct MatrixIpp *******buffer);
	void deallocate_buffer(int n_liv_pir, int n_frame, int sx_dx, int n_orient, int n_phase, struct MatrixIpp *******buffer);
	void allocate_for_energy(struct MatrixIpp ****ris, int n_liv, int n_orient,  int n_phase, int n_vel, int m, int n);
	void deallocate_for_energy(struct MatrixIpp ****ris, int n_liv, int n_orient, int n_vel, int n_phase);
	void allocate_for_decomp_energy(struct MatrixIpp *****ris, int n_liv, int n_orient, int n_phase, int m, int n);
	void deallocate_for_decomp_energy(struct MatrixIpp *****ris, int n_liv, int n_orient, int n_phase);
	void allocate_for_population(struct MatrixIpp ***ris, int n_liv, int n_orient, int m, int n);
	void deallocate_for_population(struct MatrixIpp ***ris, int n_liv, int n_orient);
	void allocate_for_decomposition_of_population(struct MatrixIpp ****ris, int n_liv, int n_orient, int m, int n);
	void deallocate_for_decomposition_of_population(struct MatrixIpp ****ris, int n_liv, int n_orient);
	void allocate_for_fulldisparity(struct MatrixIpp ***ris, int n_liv, int m, int n);
	void deallocate_for_fulldisparity(struct MatrixIpp ***ris, int n_liv);
	void allocate_for_pyramid(struct MatrixIpp **ris, int n_liv, int m, int n);
	void deallocate_for_pyramid(struct MatrixIpp **pir, int n_liv);
	//----pyramid------------------------------------------------------------------------------------------------------------------
	void pyramid_no_alloc(struct MatrixIpp image, struct MatrixIpp fil_a, struct MatrixIpp fil_b, struct MatrixIpp **pir, int n_level, const char *conv_type);
	void copy_matrixipp_no_alloc(struct MatrixIpp sorg, struct MatrixIpp *dest);
	void padding_alloc(struct MatrixIpp sorg, struct MatrixIpp *dest, int m_fil, int n_fil, const char *type);
	void zero_padding_alloc(struct MatrixIpp sorg, struct MatrixIpp *dest, int m_fil, int n_fil);
	void repeat_padding_alloc(struct MatrixIpp sorg, struct MatrixIpp *dest, int m_fil, int n_fil);
	void convolution2D_valid_allocr(struct MatrixIpp matrix, struct MatrixIpp filter, struct MatrixIpp *result);
	void convolution2D_valid_allocc(struct MatrixIpp matrix, struct MatrixIpp filter, struct MatrixIpp *result);
	void dimez_matrixipp_no_alloc(struct MatrixIpp sorg, struct MatrixIpp *dest);
	//------spatial filtering--------------------------------------------------------------------------------------------------------
	void filt_spatial_gabor_no_alloc(struct MatrixIpp imm, struct MatrixIpp *fil, struct MatrixIpp ****ris, const char *conv_type);
	static const int DX = 0;
	static const int SX = 1;
	void convolution2D_valid_no_allocr(struct MatrixIpp matrix, struct MatrixIpp filter, struct MatrixIpp *result);
	void convolution2D_valid_no_allocc(struct MatrixIpp matrix, struct MatrixIpp filter, struct MatrixIpp *result);
	void addition_matrixipp_no_alloc(struct MatrixIpp a, struct MatrixIpp b, struct MatrixIpp *c);
	void subtraction_matrixipp_no_alloc(struct MatrixIpp a, struct MatrixIpp b, struct MatrixIpp *c);
	//------shift_in_phase--------------------------------------------------------------------------------------------------------------
	void shift_in_phase(float ph_extr,float ph_step,struct MatrixIpp ***imm_filt ,  struct MatrixIpp ****ris, int n_orient);
	void mul_with_constant_matrixipp_no_alloc(struct MatrixIpp sorg, struct MatrixIpp *dest, Ipp32f value);
	void subtraction_matrixipp_inplace(struct MatrixIpp *c, struct MatrixIpp a);
	void addition_matrixipp_inplace(struct MatrixIpp *c, struct MatrixIpp a);
	//------energy_no_alloc--------------------------------------------------------------------------------------------------------------
	void energy_no_alloc(struct MatrixIpp ****imm, struct MatrixIpp ***energy, struct MatrixIpp ****dec_energy, int n_orient, int n_phase, Ipp32f th);
	void sqr_matrixipp_no_alloc(struct MatrixIpp sorg, struct MatrixIpp *dest);
	void max_of_matrixipp(struct MatrixIpp matrix, Ipp32f *valmax);
	void div_with_costant_matrixipp_inplace(struct MatrixIpp *matrix, Ipp32f value);
	void threshold_LTVal_inplace(struct MatrixIpp *matrix, Ipp32f threshold, Ipp32f value);
	void threshold_GTVal_inplace(struct MatrixIpp *matrix, Ipp32f threshold, Ipp32f value);  
	void and_matrixipp_inplace(struct MatrixIpp *c, struct MatrixIpp a);
	void mul_matrixipp_inplace(struct MatrixIpp *a, struct MatrixIpp b);
	void sqrt_matrixipp_inplace(struct MatrixIpp *matrix);
	//------warping--------------------------------------------------------------------------------------------------------------
	static const int Vx = 0;
	static const int Vy = 1;
	void warping_frame_filt_gabor(struct MatrixIpp ****buf, struct MatrixIpp *****buf_copy, struct MatrixIpp *full_disp, int n_frame, int n_orient,int n_phase);
	void warping(struct MatrixIpp sorg, struct MatrixIpp dx, struct MatrixIpp dy, struct MatrixIpp *dest);
	//---------population--------------------------------------------------------------------------------------------------------------------
	void population_no_alloc(struct MatrixIpp **energy, struct MatrixIpp disparity, struct MatrixIpp **population, struct MatrixIpp **mask, int n_orient, int n_phase);
	void decomposition_of_population_no_alloc(struct MatrixIpp *population, struct MatrixIpp ***decompose_population, int n_orient);
	//---------fulldisparity--------------------------------------------------------------------------------------------------------------------
	void fulldisparity_no_alloc(struct MatrixIpp **dec_population, struct MatrixIpp **fulldisparity, struct MatrixIpp *mask,struct MatrixIpp *mask2, int n_orient, int nc_min);
	void expand_mulX2_fulldisparity_no_alloc(struct MatrixIpp *full_in, struct MatrixIpp **full_out);
	void expand_mask(struct MatrixIpp mask, struct MatrixIpp *mask_out, int m, int n);
	void mul_with_costant_matrixipp_inplace(struct MatrixIpp *matrix, Ipp32f value);
	void expand_no_alloc(struct MatrixIpp sorg, struct MatrixIpp *dest);
	void merge(struct MatrixIpp *full_disp ,struct MatrixIpp **fullmerged);
	void cut_invalid_at_end_flow(struct MatrixIpp mask ,struct MatrixIpp **fullmerged,int nc_min);
};
#endif