/*----------------------------------------------------------------------------------
% Authors: V. Bianchi, M. Zampetti
%          A. Canessa, M. Chessa, F. Solari
% PSPC-lab - Biophysical and Electronic Engineering Department - University of Genoa
% September 2010
------------------------------------------------------------------------------------*/
#include "PopCod_Disp2D.h"
#include <ipp.h>

void test1();
void test2();

int main(){

	test2();

	return 0;
}

void test1(){
	//"Gt11B0.0833f0.25.bin": file name of the Gabor filters
	int w=640;               //image width
	int h=480;               //image height
	int n_scale=5;           //number of scales
	int n_phase=7;	         //number of phase shifts
	float dmax=1.5f;         //max detectable disparity value
	float energy_th=0.001f;  //energy threshold
	int ori_thr=4;           //threshold on orientation (0 if not used)

	PopCod_Disp2D disp("Gt11B0.0833f0.25.bin",w,h,n_scale,n_phase,dmax,energy_th,ori_thr);

	//-------input: stereo images-------------------------------------------------------------------------
	FILE *fp;
	fp=fopen("left_640x480_d10.bin","rb");
	if (fp==NULL){		printf("Error opening file\n");		exit(1);	}	
	float *L=(float*) malloc(sizeof(float)*w*h);
	fread(L,sizeof(float),w*h,fp);
	fclose(fp);
	fp=fopen("right_640x480_d10.bin","rb");
	if (fp==NULL){		printf("Error opening file\n");		exit(1);	}
	float *R=(float*) malloc(sizeof(float)*w*h);
	fread(R,sizeof(float),w*h,fp);
	fclose(fp);
	float *dx=(float*) malloc(sizeof(float)*w*h);
	float *dy=(float*) malloc(sizeof(float)*w*h);

	//------computation of disparity maps-----------------------------------------------------------------------

	disp.Compute_maps(L,R,dx,dy);

	//------output: disparity maps-----------------------------------------------------------------------------
	fp=fopen("dx_640x480_d10.bin","wb");
	fwrite(dx,sizeof(float),w*h,fp);
	fclose(fp);
	fp=fopen("dy_640x480_d10.bin","wb");
	fwrite(dy,sizeof(float),w*h,fp);
	fclose(fp);

	//--------------------------------------------------------------------------------------------------------
	free(L);
	free(R);
	free(dx);
	free(dy);
}

void test2(){
	//"Gt11B0.0833f0.25.bin": file name of the Gabor filters
	int w=320;               //image width
	int h=240;               //image height
	int n_scale=5;           //number of scales
	int n_phase=7;	         //number of phase shifts
	float dmax=1.5f;         //max detectable disparity value (for the filters)
	float energy_th=1e-4f;  //energy threshold
	int ori_thr=4;           //threshold on orientations for decoding (nc_mim of matlab)

	PopCod_Disp2D disp("Gt11B0.0833f0.25.bin",w,h,n_scale,n_phase,dmax,energy_th,ori_thr);

	//-------input: stereo images-------------------------------------------------------------------------
	FILE *fp;
	fp=fopen("left.bin","rb");
	if (fp==NULL){		printf("Error opening file\n");		exit(1);	}	
	float *L=(float*) malloc(sizeof(float)*w*h);
	fread(L,sizeof(float),w*h,fp);
	fclose(fp);
	fp=fopen("right.bin","rb");
	if (fp==NULL){		printf("Error opening file\n");		exit(1);	}
	float *R=(float*) malloc(sizeof(float)*w*h);
	fread(R,sizeof(float),w*h,fp);
	fclose(fp);
	float *dx=(float*) malloc(sizeof(float)*w*h);
	float *dy=(float*) malloc(sizeof(float)*w*h);

	//------computation of disparity maps-----------------------------------------------------------------------

	disp.Compute_maps(L,R,dx,dy);

	//------output: disparity maps-----------------------------------------------------------------------------
	fp=fopen("dx.bin","wb");
	fwrite(dx,sizeof(float),w*h,fp);
	fclose(fp);
	fp=fopen("dy.bin","wb");
	fwrite(dy,sizeof(float),w*h,fp);
	fclose(fp);

	//--------------------------------------------------------------------------------------------------------
	free(L);
	free(R);
	free(dx);
	free(dy);
}
